@@ -21,8 +21,9 @@ class Uploader(object):
 
     def __init__(self, settings):
         self.settings = settings
-        
-    def download_image_from_url(self, url):
+        self.conn = None
+
+    def download_image_from_url(self, url, hash=None):
         """downloads and saves an image from given url
         returns the MD5 name of the file needed to upload to S3
         """
@@ -30,8 +31,8 @@ class Uploader(object):
         response = urllib2.urlopen(url)
 
         content = response.read()
-
-        hash = str(md5.new(url + str(time.time())).hexdigest())
+        if hash is None:
+            hash = str(md5.new(url + str(time.time())).hexdigest())
         download_file_name = self.settings['TEMP_UPLOAD_DIR'] + '/' + hash
 
         fd = os.open(download_file_name, os.O_RDWR|os.O_CREAT)
@@ -86,11 +87,14 @@ class Uploader(object):
             im = background
             #im = im.convert("RGB")
 
+        logging.debug("create_images_for_S3 image_infos: %s " % image_infos)
+        file_names = []
         for image_info in image_infos:
             # convert to thumbnail image
             # imp = PilImage.new("RGB", im.size, (255,255,255,255))
             #imp.paste(im)
             # [([WIDTH], [HEIGHT]), [PIL FORMAT], [POSTFIX], [EXTENSION]]
+            logging.debug("create_images_for_S3  image_info: %s " % image_info)
             if image_info[0] != None:
                 width = image_info[0][0]
                 height = image_info[0][1]
@@ -109,33 +113,19 @@ class Uploader(object):
                         # use width given and scale height
                         height = im_height * (width * 1000 / im_width) / 1000
 
-                    logging.debug("original height: %s" % im_height)
-                    logging.debug("given height:    %s" % image_info[0][1])
-                    logging.debug("new height:      %s" % height)
-
-                    logging.debug("original width:  %s" % im_width)
-                    logging.debug("given width:     %s" % image_info[0][0])
-                    logging.debug("new width:       %s" % width)
                     nim = im.resize((width, height), PilImage.ANTIALIAS)
-                    gevent.sleep(0)
                     nim.save( "%s/%s%s.%s" % (file_path, file_name,image_info[2], image_info[3]), image_info[1])
-                    gevent.sleep(0)
                 else:
                     # thumb
                     im.thumbnail((width, height), PilImage.ANTIALIAS)
-                    gevent.sleep(0)
-                    im.save( "%s/%s%s.%s" % (file_path, file_name,image_info[2], image_info[3]), image_info[1])
-                    gevent.sleep(0)
             else:
                 # full size
-                gevent.sleep(0)
                 im.save( "%s/%s%s.%s" % (file_path, file_name,image_info[2], image_info[3]), image_info[1])
-                gevent.sleep(0)
-
+            logging.debug("filename created: %s%s.%s" % (file_name, image_info[2], image_info[3]))
             file_names.append("%s%s.%s" % (file_name, image_info[2], image_info[3]))
 
-        logging.debug(file_names)
-        return file_names    
+        logging.debug("create_images_for_S3 return filenames: %s " % file_names)
+        return file_names
 
     def _alpha_composite(self, src, dst):
         """places a background in place of transparancy.
@@ -151,17 +141,27 @@ class Uploader(object):
         gevent.sleep(0)
         return dst
 
+    def get_connection_s3(self):
+        """ create our s3 connection or return it if exists""" 
+        if self.conn is None:
+            key = self.settings["AMAZON_KEY"]
+            secret = self.settings["AMAZON_SECRET"]
+            bucket_name = self.settings["AMAZON_BUCKET"]
+            host = self.settings["S3_HOST"]
+            port = self.settings["S3_PORT"]
+            self.conn = boto.connect_s3(key, secret, host = host, port = port)
+        return self.conn
+
     def upload_to_S3(self, file_name):
         """upload a file to S3 and return the path name"""
         logging.debug("upload_to_S3 : %s" % file_name)
 
         file_path = self.settings['TEMP_UPLOAD_DIR']
         file_names = self.create_images_for_S3(file_path, file_name)
-
-        key = self.settings["AMAZON_KEY"]
-        secret = self.settings["AMAZON_SECRET"]
+        logging.debug("file_names: %s" % file_names)
+        logging.debug("len(file_names): %s" % len(file_names))
         bucket_name = self.settings["AMAZON_BUCKET"]
-        conn = boto.connect_s3(key, secret)
+        conn = self.get_connection_s3()
 
         if "INIT_BUCKET" not in self.settings or self.settings["INIT_BUCKET"] == False:
             # bucket must exist
@@ -193,13 +193,16 @@ class Uploader(object):
             """ % bucket_name)
 
         images = self.create_images_for_S3(file_path, file_name)
+        logging.debug("file_names: %s" % file_names)
+        logging.debug("images: %s" % images)
         for image_file_name in file_names:
+            logging.debug("image_file_name: %s" % image_file_name)
             logging.debug( 'Uploading %s to Amazon S3 bucket %s' % (image_file_name, bucket_name))
             k = Key(bucket)
             k.key = image_file_name
             k.set_contents_from_filename("%s/%s" % (file_path, image_file_name))
             acl = 'public-read'
-            logging.debug("acl: %s" % acl)
+            #logging.debug("acl: %s" % acl)
             k.set_acl(acl)
 
         return True
@@ -210,10 +213,8 @@ class Uploader(object):
         """
         logging.debug("delete_from_S3 : %s" % file_name)
 
-        key = self.settings["AMAZON_KEY"]
-        secret = self.settings["AMAZON_SECRET"]
         bucket_name = self.settings["AMAZON_BUCKET"]
-        conn = boto.connect_s3(key, secret)
+        conn = self.get_connect_s3()
         bucket = conn.get_bucket(bucket_name)
         image_infos = self.settings['IMAGE_INFO']
 
@@ -232,3 +233,4 @@ class Uploader(object):
         bucket.delete_key(k)
 
         return True
+
