@@ -10,7 +10,12 @@ import re
 import urllib2
 from BeautifulSoup import BeautifulSoup
 from brubeck.auth import authenticated
-from brubeck.request_handling import MessageHandler, WebMessageHandler, JSONMessageHandler
+from brubeck.request_handling import (
+    MessageHandler,
+    WebMessageHandler,
+    JSONMessageHandler,
+)
+from brubeckservice.base import ServiceMessageHandler
 from brubeckuploader.base import Uploader
 from math import log
 from PIL import Image as PILImage
@@ -21,7 +26,7 @@ from urlparse import urlparse
 ## This should be in Brubeck soon
 ##
 def lazyprop(method):
-    """ A nifty wrapper to only load preoperties when accessed
+    """ A nifty wrapper to only load properties when accessed
     uses the lazyProperty pattern from: 
     http://j2labs.tumblr.com/post/17669120847/lazy-properties-a-nifty-decorator
     inspired by  a stack overflow question:
@@ -40,7 +45,7 @@ def lazyprop(method):
             if getattr(self, attr_name) == 'undefined':
                 setattr(self, attr_name, None)
         return getattr(self, attr_name)
-    return _lazyprop    
+    return _lazyprop
 
 
 ##
@@ -53,12 +58,24 @@ class BrubeckUploaderBaseHandler(MessageHandler):
     def settings(self):
         """get our settings
         """
-        return self.application.get_settings('uploader')
+        try:
+            return self.application.get_settings('uploader')
+        except:
+            pass
+        return None
 
     @lazyprop
     def uploader(self):
         return Uploader(self.settings)
 
+    @lazyprop
+    def echo_parameters(self):
+        """ a list of query string parameters to echo back in the response """
+        echo_parameters = self.message.get_argument('echo_parameters', '')
+        echo_parameters =  echo_parameters.split(',')
+        logging.debug(echo_parameters)
+        return echo_parameters
+
     def human_readable_file_size(self, num):
         """Human friendly file size"""
         unit_list = zip(['bytes', 'kB', 'MB', 'GB', 'TB', 'PB'], [0, 0, 1, 2, 2, 2])
@@ -88,43 +105,50 @@ class BrubeckUploaderBaseHandler(MessageHandler):
         self.set_status(200)
         return self.render()
 
-    def saveFile(self, file_name, is_url=False, hash=None):
+    def saveFile(self, file_name, is_url=False, hash=None, file_content=None):
         """Save an uploaded file or downloads a file from a url and places it in the TMP directory"""
         if is_url:
-            hash = self.uploader.download_image_from_url(file_name)
+            hash = self.uploader.download_image_from_url(file_name, hash)
         else:
-            hash = str(md5.new(file_name + str(time())).hexdigest())
+            if hash is None:
+                hash = str(md5.new(file_name + str(time())).hexdigest())
 
         download_file_name = self.application.project_dir + '/' + self.settings['TEMP_UPLOAD_DIR'] + '/' + hash
         fd = os.open(download_file_name, os.O_RDWR|os.O_CREAT)
 
+        if file_content is None:
+            file_content = self.message.body
         if is_url is False:
-            os.write(fd, self.message.body)
-            
+            os.write(fd, file_content)
+
         # get our mime-type
         mime = magic.Magic(mime=True)
         mime_type = mime.from_file(download_file_name)
-    
+
         logging.debug("filename: %s" % file_name)
         logging.debug("hash: %s" % hash)
         logging.debug("download_file_name: %s" % download_file_name)
         logging.debug("mime_type: %s" % mime_type)
-    
+
         logging.debug("checking mime_type: %s" % mime_type)
         if not mime_type in self.settings['ACCEPTABLE_UPLOAD_MIME_TYPES']:
             raise Exception("unacceptable mime type: %s" % mime_type)
             os.remove(download_file_name)
         logging.debug("mime_type OK")
-    
+
         width, height = PILImage.open(open(download_file_name)).size
         logging.debug("width: %s" % width)
         logging.debug("height: %s" % height)
-    
+
         message = 'The file "' + file_name + '" was uploaded successfully'
-    
-        file_size = 100
+
+        file_size = os.fstat(fd).st_size
         human_readable_file_size = self.human_readable_file_size(file_size)
-    
+
+        # spit back our parameters sent with the file
+        for param_name in self.echo_parameters:
+            self.add_to_payload(param_name, self.message.get_argument(param_name, ''))
+
         self.add_to_payload('success', True)
         self.add_to_payload('message', message)
         self.add_to_payload('filename', file_name)
@@ -135,7 +159,7 @@ class BrubeckUploaderBaseHandler(MessageHandler):
         self.add_to_payload('width', width)
         self.add_to_payload('height', height)
         self.set_status(200)
-        
+
 
 class TemporaryImageViewHandler(WebMessageHandler, BrubeckUploaderBaseHandler):
     """this is built to be compatible with fileuploader.js"""
@@ -148,8 +172,8 @@ class TemporaryImageViewHandler(WebMessageHandler, BrubeckUploaderBaseHandler):
             requested_file_name = self.application.project_dir + '/' + self.settings['TEMP_UPLOAD_DIR'] + '/' + file_name
 
             fp = open(requested_file_name)
-            file_contents =  fp.read()
-        
+            file_content =  fp.read()
+
             # get our mime-type
             mime = magic.Magic(mime=True)
             mime_type = mime.from_file(requested_file_name)
@@ -160,9 +184,9 @@ class TemporaryImageViewHandler(WebMessageHandler, BrubeckUploaderBaseHandler):
             logging.debug("mime_type: %s" % mime_type)
 
             self.set_status(200)
-            self.set_body(file_contents)
+            self.set_body(file_content)
             self.headers['Content-Type'] = mime_type
-            self.headers['Content-Length'] = len(file_contents)
+            self.headers['Content-Length'] = len(file_content)
 
         except Exception as e:
             raise
@@ -179,14 +203,14 @@ class TemporaryImageUploadHandler(JSONMessageHandler, BrubeckUploaderBaseHandler
         """upload a temporary files"""
         logging.debug("TemporaryImageUploadHandler post")
         try:
-            qqfile = self.get_argument('qqfile', None)
-            file_contents = self.message.body
+            qqfile = self.message.get_argument('qqfile', None)
+
+            file_content = self.message.body
             logging.debug(self.settings)
-            if len(file_contents) > 0:
+            if len(file_content) > 0:
                 fn = qqfile
                 self.saveFile(fn, is_url=False)
 
-    
             else:
                 raise Exception('No file was uploaded')
 
@@ -205,7 +229,7 @@ class TemporaryImageFromURLUploadHandler(JSONMessageHandler, BrubeckUploaderBase
 
     @lazyprop
     def fetch_image_url(self):
-        fetch_image_url = self.get_argument('fetch_image_url', None)
+        fetch_image_url = self.message.get_argument('fetch_image_url', None)
         return fetch_image_url
 
     def post(self):
@@ -244,8 +268,7 @@ class ImageURLFetcherHandler(JSONMessageHandler, BrubeckUploaderBaseHandler):
                     response = urllib2.urlopen(fetch_image_url)
                     the_page = response.read()
                     pool = BeautifulSoup(the_page)
-                    base_url = self.get_base_url(fetch_image_url)
-                    image_urls += (self.get_url_images(pool, base_url))
+                    image_urls += (self.get_url_images(pool, fetch_image_url))
 
                 except Exception as e:
                         logging.debug('Unable to fetch images for %s' % fetch_image_url);
@@ -262,7 +285,7 @@ class ImageURLFetcherHandler(JSONMessageHandler, BrubeckUploaderBaseHandler):
         self.set_status(200)
         return self.render()
 
-    def get_url_images(self, pool, base_url):
+    def get_url_images(self, pool, fetch_image_url):
         """get images urls from a BeatifulSoup 'pool'"""
         image_urls = []
         tags = pool.findAll('meta',
@@ -300,7 +323,7 @@ class ImageURLFetcherHandler(JSONMessageHandler, BrubeckUploaderBaseHandler):
                     logging.debug('img_src tag: %s' % tag)
                     url = self.screen_and_fix_url(
                             _get_tag_attr(tag, 'img_src'),
-                            base_url
+                            fetch_image_url
                         )
                     if url is not None:
                         image_urls.append(url)
@@ -312,7 +335,7 @@ class ImageURLFetcherHandler(JSONMessageHandler, BrubeckUploaderBaseHandler):
                 logging.debug('img tag: %s' % tag)
                 url = self.screen_and_fix_url(
                         _get_tag_attr(tag, 'src'),
-                        base_url
+                        fetch_image_url
                     )
                 image_urls.append(url)
 
@@ -323,13 +346,18 @@ class ImageURLFetcherHandler(JSONMessageHandler, BrubeckUploaderBaseHandler):
         parsed_url = urlparse(page_url)
         return "%s://%s" % (parsed_url[0], parsed_url[1])
 
-    def screen_and_fix_url(self, url, base_url):
-        logging.debug("screen_and_fix_url('%s', '%s')" % (url, base_url))
+    def get_base_path(self, page_url):
+        """get the base url from a url"""
+        parsed_url = urlparse(page_url)
+        return "%s://%s" % (parsed_url[0], parsed_url[1])
+
+    def screen_and_fix_url(self, url, fetch_image_url):
+        logging.debug("screen_and_fix_url('%s', '%s')" % (url, fetch_image_url))
         # first screen it
         if url is None or self.screen_url(url) == False:
             return None
         # and if needed fix it
-        return self.fix_url(url, base_url)
+        return self.fix_url(url, fetch_image_url)
 
     def screen_url(self, url):
         """we don't want things like ads"""
@@ -337,18 +365,71 @@ class ImageURLFetcherHandler(JSONMessageHandler, BrubeckUploaderBaseHandler):
             return False;
         return True
 
-    def fix_url(self, url, base_url):
+    def fix_url(self, url, fetch_image_url):
         """Give a url an absolute path if it does not have one"""
-        logging.debug("fix_url('%s', '%s')" % (url, base_url))
+        logging.debug("fix_url('%s', '%s')" % (url, fetch_image_url))
+        base_url = self.get_base_url(fetch_image_url)
+        base_path = self.get_base_path(fetch_image_url)
         if url[0:4] == 'http':
             return url
-        parse_url = urlparse(base_url)
-        if url[0:1] == '/':
+        if url[0:2] == '//':
+            url = "%s:%s" % (urlparse(base_url)[0], url)
+        elif url[0:1] == '/':
             url = "%s%s" % (base_url, url)
-        elif parse_url[2] == '':
+        elif base_url[2] == '':
             url = "%s/%s" % (base_url, url)
         else:
             path_parts = url.split('/')
             path_parts.pop()
-            url = "%s/%s/" % (base_url, path_parts.join('/'), url)
+            url = "%s/%s/%s" % (base_url, self.join_list(path_parts, '/'), url)
         return url
+
+    def join_list(self, l, d = ''):
+        """Joins a list like string.join"""
+        logging.debug("join_list('%s', '%s')" % (l, d))
+        return d.join(v for v in l)
+
+
+class UploadHandler(ServiceMessageHandler, BrubeckUploaderBaseHandler):
+    """A sevice to upload an image, process it and push it to S3"""
+
+    def put(self):
+        logging.debug("UploadHandler put()")
+
+        try:
+            # save the file
+            file_content = self.message.get_argument('file_content', '').decode('base64')
+            file_name = self.message.get_argument('file_name', None)
+            logging.debug("file_name: %s" % file_name)
+            hash = self.message.get_argument('hash', None)
+            if self.settings is None:
+                self._settings = self.message.get_argument('settings', '')
+
+            if len(file_content) > 0 and len(self._settings) > 0:
+                self.saveFile(file_name, 
+                    is_url=False, 
+                    hash=hash, 
+                    file_content=file_content
+                )
+            
+                if self.uploader.upload_to_S3(hash):
+                    # success
+                    self.set_status(200, "Uploaded to S3!")
+                else:
+                    self.set_status(500, "Failed to upload to S3!")
+
+                self.add_to_payload("file_name", file_name)
+                self.add_to_payload("settings_image_info", self.settings["IMAGE_INFO"])
+
+            else:
+                raise Exception('No file was uploaded')
+
+        except Exception as e:
+            logging.debug(e.message)
+            self.set_status(500)
+            self.add_to_payload('error', e.message)
+            raise
+
+
+        self.headers = {"METHOD": "response"}
+        return self.render()
